﻿/*
 * Copyright (c) 2015, Jason Maltzen

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
*/

using System;
using System.Collections.Generic;

namespace DesertPaintLab
{
    public class ReactionSet
    {
        // ingredient -> [ingredient, reaction]
        SortedDictionary<string, SortedDictionary<string, Reaction>> reactions =
            new SortedDictionary<string, SortedDictionary<string, Reaction>>();

        public ReactionSet()
        {
        }

        public Reaction Find(Reagent reagent1, Reagent reagent2)
        {   
            SortedDictionary<string, Reaction> secondReagentDict = null;
            Reaction reaction = null;
            reactions.TryGetValue(reagent1.PracticalPaintName, out secondReagentDict);
            if (secondReagentDict != null)
            {
                secondReagentDict.TryGetValue(reagent2.PracticalPaintName, out reaction);
            }
            return reaction;
        }
        
        public void Set(Reagent reagent1, Reagent reagent2, Reaction reaction)
        {
            SortedDictionary<string, Reaction> secondReagentDict = null;
            reactions.TryGetValue(reagent1.PracticalPaintName, out secondReagentDict);
            if (secondReagentDict == null)
            {
                secondReagentDict = new SortedDictionary<string, Reaction>();
                reactions.Add(reagent1.PracticalPaintName, secondReagentDict);
            }
            secondReagentDict[reagent2.PracticalPaintName] = reaction;
        }

        public void Remove(Reagent reagent1, Reagent reagent2)
        {
            SortedDictionary<string, Reaction> secondReagentDict = null;
            reactions.TryGetValue(reagent1.PracticalPaintName, out secondReagentDict);
            if (secondReagentDict != null)
            {
                secondReagentDict.Remove(reagent2.PracticalPaintName);
            }
        }

        public void Clear()
        {
            reactions.Clear();
        }
    }
}

